import sys, Crime, optparse, time
from twisted.internet import reactor
from parsedatetime import parsedatetime

def parseDatetimeOption(option, opt, value, parser):
    struct_time, flag = parsedatetime.Calendar().parse(value)
    
    if flag == 1:
        setattr(parser.values, option.dest, time.strftime('%Y%m%d', struct_time))
        
        if option.dest == 'dateStart' and parser.values.dateEnd is None:
            # dateEnd defaults to same as dateStart
            setattr(parser.values, 'dateEnd', time.strftime('%Y%m%d', struct_time))

        return

    raise optparse.OptionValueError('Please provide a date, e.g. "yesterday" or "Nov 16, 1977" (not "%s")' % value)

def parseCrimeOption(option, opt, value, parser):
    crimes = Crime.Image.crimes()
    
    if crimes.has_key(value.upper()):
        setattr(parser.values, option.dest, value.upper())
        return
    else:
        names = [name for (name, crime) in crimes.items()
                 if crime.name.lower() == value.lower()]
        if len(names) == 1:
            setattr(parser.values, option.dest, names[0])
            return

    raise optparse.OptionValueError('Please provide a valid crime, e.g. "Robbery" or "Simple Assault" (not "%s")' % value)

def parseDistrictOption(option, opt, value, parser):
    districts = Crime.Geocode.districts()
    
    if districts.has_key(value.upper()):
        setattr(parser.values, option.dest, value.upper())
        return
    else:
        names = [name for (name, district) in districts.items()
                 if district.name.lower() == value.lower()]
        if len(names) == 1:
            setattr(parser.values, option.dest, names[0])
            return

    raise optparse.OptionValueError('Please provide a valid district, e.g. "Council District 1" (not "%s")' % value)

parser = optparse.OptionParser()

parser.add_option('-d', '--database', type='string', dest='dataStore',
                  help='Database file to save to, default ":memory:"',
                  action='store', default=':memory:')

parser.add_option('-a', '--area', type='string', dest='areaName',
                  help='Area to cover (council district), omit for all. Example: "council district 3".',
                  action='callback', callback=parseDistrictOption)

parser.add_option('-c', '--crime', type='string', dest='crimeType',
                  help='Type of crime, omit for all. Example: "arson".',
                  action='callback', callback=parseCrimeOption)

parser.add_option('-s', '--start', type='string', dest='dateStart',
                  help='Start date. Example: "2 days ago".', default=time.strftime('%Y%m%d', parsedatetime.Calendar().parse('yesterday')[0]),
                  action='callback', callback=parseDatetimeOption)

parser.add_option('-e', '--end', type='string', dest='dateEnd',
                  help='End date. Example: "Nov 16, 2006"',
                  action='callback', callback=parseDatetimeOption)

if __name__ == '__main__':
    options = parser.parse_args()[0].__dict__
    options['dataStore'] = Crime.Store.getDB(options['dataStore'])
    agent = Crime.Agent.CrimeAgent(**options)
    reactor.callWhenRunning(agent.queueRequests)
    reactor.run()